/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/binding/CommandBinding.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli.binding;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli2.CommandLine;
import org.apache.commons.cli2.option.Command;

import com.netapp.common.cli.validator.ValidationException;

/**
 * A binding between an ApacheCLI Command object and the Java Method to which that command corresponds
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class CommandBinding implements BindingContainer {
    private Method method;
    private Command baseCommand;
    private Command subCommand;
    private CommandLine commandLine;
    private List<ParameterBinding> parameterBindings = new ArrayList<ParameterBinding>();
    private GlobalBindings globalBindings;
    
    public void setMethod(Method method) {
        this.method = method;
    }
    
    public Method getMethod() {
        return this.method;
    }
    
    public void setCommandLine(CommandLine commandLine) {
        this.commandLine = commandLine;
    }
    
    public CommandLine getCommandLine() {
        return this.commandLine;
    }
    
    public void setBaseCommand(Command command) {
        this.baseCommand = command;
    }
    
    public Command getBaseCommand() {
        return this.baseCommand;
    }
    
    public void setSubCommand(Command subCommand) {
        this.subCommand = subCommand;
    }
    
    public Command getSubCommand() {
        return this.subCommand;
    }
    
    public List<ParameterBinding> getParameterBindings() {
        return this.parameterBindings;
    }
    
    @Override
    public String toString() {
        return "Command[" + this.baseCommand.getPreferredName() + "("  
            + (this.subCommand == null ? "no subcommand" : this.subCommand.getPreferredName()) + ")" 
            + "] Method[" + this.method.getName() + "] Params" + this.parameterBindings;  
    }
    
    public void setGlobalFields(Object object) throws ValidationException {
        if (this.globalBindings != null) {
            this.globalBindings.hydrate(object, this.commandLine);
        }        
    }
    
    public Object execute(Object object) throws InvocationTargetException, ValidationException {
        try {
            /*
             * First set global bindings on object
             */
            if (this.globalBindings != null) {
                this.globalBindings.hydrate(object, this.commandLine);
            }
            
            /*
             * Now invoke the method with appropriate args
             */
            List<Object> args = new ArrayList<Object>();
            for (ParameterBinding parameterBinding : this.parameterBindings) {
                args.add(parameterBinding.hydrate(this.commandLine));
            }
            return this.method.invoke(object, args.toArray());
        }
        catch (IllegalArgumentException e) {
            throw new RuntimeException(e);
        }
        catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }
    
    public class ParameterBinding {
        private int parameterNumber;
        private ObjectBinding objectBinding;
        
        public ParameterBinding(int parameterNumber, ObjectBinding objectBinding) {
            this.parameterNumber = parameterNumber;
            this.objectBinding = objectBinding;
            CommandBinding.this.parameterBindings.add(this);
        }
        
        public void setParameterNumber(int paramNum) {
            this.parameterNumber = paramNum;
        }
        
        public Object hydrate(CommandLine commandLine) throws ValidationException {
            return this.objectBinding.hydrate(commandLine);
        }
        
        @Override
        public String toString() {
            return this.parameterNumber + "=" + this.objectBinding;
        }
    }
    
    @SuppressWarnings("unchecked") // Unchecked casts on the returns
    static <T> T convertToType(Class<T> typeClass, String s) {
        if (typeClass.equals(Integer.class) || typeClass.equals(int.class)) {
            return (T) Integer.valueOf(s);
        }
        else if (typeClass.equals(Boolean.class) || typeClass.equals(boolean.class)) {
            return (T) Boolean.valueOf(s == null ? false : true);
        }
        else {
            return (T) s;
        }
    }

    public void setGlobalBindings(GlobalBindings globalBindings) {
        this.globalBindings = globalBindings;
    }
}