/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/binding/PrimitiveObjectBinding.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli.binding;

import java.lang.reflect.Array;
import java.util.List;

import org.apache.commons.cli2.CommandLine;
import org.apache.commons.cli2.Option;

import com.netapp.common.MsgKey;
import com.netapp.common.cli.CliParameter;
import com.netapp.common.cli.enumeration.CliEnumTranformerFactory;
import com.netapp.common.cli.enumeration.CliEnumTransformer;
import com.netapp.common.cli.validator.ValidationException;
import com.netapp.common.cli.validator.ValidatorType;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.ClassUtils;

public class PrimitiveObjectBinding extends ObjectBinding {
    private static final NALogger logger = NALogger.getLogger(PrimitiveObjectBinding.class);
    
    public PrimitiveObjectBinding(Class<?> type, CliParameter cliParameter, Option option) {
        this(type, cliParameter, option, false);
    }
    
    public PrimitiveObjectBinding(Class<?> type, CliParameter cliParameter, Option option, boolean parentIsArray) {
        super(type, cliParameter, option);
        
        setParentIsArray(parentIsArray);
    }
    
    public String getRawValue(CommandLine commandLine) {
        return (String) commandLine.getValue(this.getOption());
    }
    
    @SuppressWarnings("unchecked") // for commandLine.getValues call.
    @Override 
    public Object doHydrate(CommandLine commandLine) throws ValidationException {
        Class classType = getObjectClass();
        
        if (null != classType.getComponentType()) {
            classType = classType.getComponentType();
        }
        
        if (getObjectClass().equals(Boolean.class) || getObjectClass().equals(boolean.class)) {
            return commandLine.hasOption(this.getOption());
        }
        else if (isParentIsArray()) {
            // we have to allow a List<String> of values
            List<String> values = commandLine.getValues(getOption());
            
            Object arrayInstance = Array.newInstance(classType, values.size());
            
            for (int i = 0; i < values.size(); i++) {
                String value = values.get(i);
                
                Object convertedValue;
                if (classType.isEnum()) {
                    convertedValue = hydrateEnum(value, classType);
                }
                else {
                    convertedValue = ClassUtils.createObject(classType, value);
                }
                
                Array.set(arrayInstance, i, convertedValue);
            }
                        
            return arrayInstance;
        }
        else if (classType.isEnum()) {
            return hydrateEnum(this.getRawValue(commandLine), classType);
        }
        else {
            String s = this.getRawValue(commandLine);
            // logger.internal("Option " + this.getOption() + " Converting " + s + " to " + type );
            try {
                return ClassUtils.createObject(classType, s);
            }
            catch (IllegalArgumentException iae) {
                //System.out.println(getOptionDisplay());
                logger.error(MsgKey.CLI_BASE_NUMERIC_PARAMETER_REQUIRED, s, getOptionDisplay());
                logger.internal(iae.getLocalizedMessage());
                throw new ValidationException(getOptionDisplay());
            }
        }
    }

    private <E extends Enum<E>> E hydrateEnum(String value, Class<E> classType) {
        CliEnumTransformer cliEnumTransformer = CliEnumTranformerFactory.createCliEnumTransformer(getCliParameter().enumTransformer());
        return cliEnumTransformer.cliOptionToEnumValue(classType, value);
    }

    @Override
    protected void validate(CommandLine commandLine) throws ValidationException {
        for (ValidatorType validatorType : this.getCliParameter().validators()) {
            validatorType.getValidator().validate(this, commandLine);
        }
    }
    
    @Override
    public String toString() {
        return this.getOption().toString();
    }
}
