/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/execution/ShellExecutorFactory.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import static com.netapp.common.platform.linux.OperatingSystemImpl.*;

import com.netapp.common.platform.OperatingSystem;


/**
 * Factory class for creating {@link ShellExecutor} objects
 * capable of executing {@link ShellCommand}s on this operating system 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ShellExecutorFactory {

    private static OperatingSystem operatingSystem;
    
    private ShellExecutorFactory() {
    }
    
    private static OperatingSystem getOperatingSystem() {
        if (operatingSystem == null) {
            operatingSystem = OperatingSystem.createInstance();
        }
        return operatingSystem;
    }

    public static ShellExecutor createShellExecutor() {
        return createShellExecutor(getOperatingSystem());
    }
    
    static ShellExecutor createShellExecutor(OperatingSystem operatingSystem) {
        if (operatingSystem.isWindows()) {
            return new WindowsShellExecutor(new WindowsShellCommandTransformer());
        }
        else {
            return new DefaultShellExecutor(new UnixShellCommandTransformer());  
        }
    }
    
    public static ShellExecutor createUserSwitchingShellExecutor(String user) {
        OperatingSystem os = getOperatingSystem();
        
        if (os.isWindows()) {
            throw new UnsupportedOperationException();
        }
        else {
            return new UnixUserSwitchingShellExecutor(createUserSwitchingShellCommandTransformer(user), user);
        }
    }
    
    private static ShellCommandTransformer createUserSwitchingShellCommandTransformer(String user) {
        return createUserSwitchingShellCommandTransformer(user, null);
    }
    
    private static ShellCommandTransformer createUserSwitchingShellCommandTransformer(String user, ShellCommandTransformer shellCommandTransformer) {
        OperatingSystem os = getOperatingSystem();
        
        if (os.isWindows()) {
            throw new UnsupportedOperationException();
        }
        else if (LINUX_PLATFORM_NAME.equals(os.getPlatform())) {
            return new LinuxUserSwitchingShellCommandTransformer(shellCommandTransformer, user);    
        }
        else {
            return new UnixUserSwitchingShellCommandTransformer(shellCommandTransformer, user);
        }
    }
    
    public static ShellExecutor createSshShellExecutor(String host) {
        ShellCommandTransformer shellCommandTransformer = createSshShellCommandTransformer(host);
        return createSshExecutor(shellCommandTransformer);
    }

    private static ShellExecutor createSshExecutor(ShellCommandTransformer shellCommandTransformer) {
        OperatingSystem os = getOperatingSystem();
        
        if (os.isWindows()) {
            throw new UnsupportedOperationException();
        }
        else {
            return new DefaultShellExecutor(shellCommandTransformer);
        }
    }

    private static ShellCommandTransformer createSshShellCommandTransformer(String host) {
        return createSshShellCommandTransformer(host, null);
    }

    private static ShellCommandTransformer createSshShellCommandTransformer(String host, ShellCommandTransformer shellCommandTransformer) {
        OperatingSystem os = getOperatingSystem();
        
        if (os.isWindows()) {
            throw new UnsupportedOperationException();
        }
        else {
            return new UnixSshShellCommandTransformer(shellCommandTransformer, host);
        }
    }
    
    public static ShellExecutor createUserSwitchingSshShellExecutor(String user, String host) {
        ShellCommandTransformer sshShellCommandTransformer = createSshShellCommandTransformer(host);
        ShellCommandTransformer userSwitchingSshShellCommandTransformer = createUserSwitchingShellCommandTransformer(user, sshShellCommandTransformer);
        
        return createSshExecutor(userSwitchingSshShellCommandTransformer);
    }
}
