/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/RegisterTemplate.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import static com.netapp.common.flow.FlowConstants.*;

import java.lang.reflect.Type;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.ClassUtils;
import com.netapp.common.util.StringUtils;
import com.netapp.common.xml.DOMUtils;

class RegisterTemplate implements Comparable<RegisterTemplate> {
    private static final NALogger logger = NALogger.getLogger(RegisterTemplate.class);

    private String name;
    private Type type;
    private String inputName;
    private String outputName;

    public RegisterTemplate(String name, Type type) {
        this.name = name;
        this.type = type;
    }
    
    public RegisterTemplate(String name, String inputName, String outputName, String typeName) {
        this.name = name;
        this.inputName = inputName;
        this.outputName = outputName;
        try {
            this.type = ClassUtils.parseTypeDeclaration(typeName);
        }
        catch (ClassNotFoundException e) {
            throw new FlowException(logger.error(MsgKey.FLOW_TYPE_CLASS_NOT_FOUND, typeName));
        }
        catch (IllegalArgumentException e) {
            throw new FlowException(logger.error(MsgKey.FLOW_TYPE_CLASS_NOT_FOUND, typeName));
        }
    }
    
    public RegisterTemplate(Node xml) {
        NamedNodeMap attrs = xml.getAttributes();
        this.name = attrs.getNamedItem(NAME_ATTRIBUTE).getNodeValue();
        
        Node inputNode = null;
        Node outputNode = null;
        Node typeNode = null;
        for (Node child = xml.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, TYPE_ELEMENT)) {
                typeNode = child;
            }
            else if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, INPUT_ELEMENT)) {
                inputNode = child;
            }
            else if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, OUTPUT_ELEMENT)) {
                outputNode = child;
            }
        }
        
        String typeName = null;
        if (attrs.getNamedItem(TYPE_ATTRIBUTE) != null) {
            typeName = attrs.getNamedItem(TYPE_ATTRIBUTE).getNodeValue();
        }
        
        if (StringUtils.isBlank(typeName) && typeNode != null) {
            typeName = typeNode.getTextContent().trim();

        }
        
        if (StringUtils.isBlank(typeName)) {
            typeName = "Object";
        }
        
        try {
            this.type = ClassUtils.parseTypeDeclaration(typeName);
        }
        catch (ClassNotFoundException e) {
            throw new FlowException(logger.error(MsgKey.FLOW_TYPE_CLASS_NOT_FOUND, typeName));
        }
        catch (IllegalArgumentException e) {
            throw new FlowException(logger.error(MsgKey.FLOW_TYPE_CLASS_NOT_FOUND, typeName));
        }

        if (inputNode != null) {
            this.inputName = DOMUtils.getAttribute(inputNode, NAME_ATTRIBUTE);
            this.inputName = StringUtils.coalesce(this.inputName, this.name);
        }
        
        if (outputNode != null) {
            this.outputName = DOMUtils.getAttribute(outputNode, NAME_ATTRIBUTE);
            this.outputName = StringUtils.coalesce(this.outputName, this.name);
        }
    }
    
    public String getName() {
        return this.name;
    }
    
    public Type getType() {
        return this.type;   
    }
    
    public boolean isInput() {
        return !StringUtils.isEmpty(this.inputName);
    }
    
    public String getInputName() {
        return this.inputName;
    }
    
    public boolean isOutput() {
        return !StringUtils.isEmpty(this.outputName);
    }
    
    public String getOutputName() {
        return this.outputName;
    }
  
    @Override
    public String toString() {
        return this.type.toString() + " " + this.name;
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode();
    }
    
    public int compareTo(RegisterTemplate o) {
        return this.name.compareTo(o.name);
    }
}