/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/tasks/diag/DefaultDiagnosticCleanup.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.tasks.diag;

import java.io.File;

import com.netapp.common.flow.Task;
import com.netapp.common.flow.TaskInput;
import com.netapp.common.logging.NALogger;

/**
 * Default cleanup task for diagnostics.
 * This removes a previously defined scratch directory and subsequent created files.
 * 
 * @author debert
 * @version $Revision: #1 $
 */

public class DefaultDiagnosticCleanup implements Task {
    private static final NALogger logger = NALogger.getLogger(DefaultDiagnosticCleanup.class);

    private String scratchPath; 
    
    private boolean remove(File file) {
        boolean removed = false;
        
        if (!file.canWrite()) {
            logger.internal("unable to delete file/directory " + file.getAbsolutePath());
        }
        
        if (file.isDirectory()) {
            for (File f : file.listFiles()) {
                removed = remove(f);
            }
        }
        
        removed = file.delete();
        
        return removed;
    }
    
    public void execute() throws Exception {
        if (scratchPath == null) {
            logger.internal("path for cleaning up is not set");
            throw new Exception("path for cleaning up is not set");
        }
        logger.internal("cleaning up scratch path " + scratchPath);
        
        boolean removed = remove(new File(scratchPath));
        
        if (removed) {
            logger.internal("scratch path " + scratchPath + " successfully removed");
        }
        else {
            logger.internal("unable to remove directory: " + scratchPath);
        }
    }
    
    @TaskInput
    public void setScratchPath(String scratchPath) {
        this.scratchPath = scratchPath;
    }

}
