/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/gwt/codegen/GWTClassConverter.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.gwt.codegen;

import java.util.LinkedList;
import java.util.List;

import javax.lang.model.element.ElementKind;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.PrimitiveType;
import javax.lang.model.type.TypeKind;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.ElementFilter;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;

import com.netapp.common.generator.NetappClassGenerator;
import com.netapp.common.util.StringUtils;

public class GWTClassConverter extends NetappClassGenerator {

    private TypeElement rootTypeElement;
    private Elements elements;
    private Types types;

    GWTClassConverter(TypeElement typeElement, Elements elements, Types types) {
        super(elements.getPackageOf(typeElement).getQualifiedName().toString());
        
        this.rootTypeElement = typeElement;
        this.elements = elements;
        this.types = types;
        
        generateClassFile();
    }
    
    private void generateClassFile() {
        List<String> extendedInterfaces = new LinkedList<String>();
        extendedInterfaces.add("com.google.gwt.user.client.rpc.IsSerializable");
        for (TypeMirror extendedInterfaceType : this.rootTypeElement.getInterfaces()) {
            if (extendedInterfaceType.toString().equals(Cloneable.class.getName())
                    || extendedInterfaceType.toString().startsWith(Comparable.class.getName())) {
                continue;
            }
            extendedInterfaces.add(extendedInterfaceType.toString());
        }

        addLine("public %sclass %s extends %s implements %s {",
            this.rootTypeElement.getModifiers().contains(Modifier.ABSTRACT) ? "abstract " : "",
            this.rootTypeElement.getSimpleName(),
            this.rootTypeElement.getSuperclass(),
            StringUtils.collectionToString(extendedInterfaces, ", ")
        );
        
        increaseIndent();
        addLine();

        this.generateBasicClassBody(this.rootTypeElement);

        for (TypeElement nestedTypeElement : ElementFilter.typesIn(this.rootTypeElement.getEnclosedElements())) {
            
            if (nestedTypeElement.getKind() == ElementKind.ENUM) {
                addLine("public enum %s {", nestedTypeElement.getSimpleName());
                
                increaseIndent();
                
                for (VariableElement field : ElementFilter.fieldsIn(nestedTypeElement.getEnclosedElements())) {
                    addLine("%s,", field.getSimpleName());
                }
                addLine(";");
                
                decreaseIndent();
                
                addLine("}");
            }
            else {
                extendedInterfaces = new LinkedList<String>();
                extendedInterfaces.add("com.google.gwt.user.client.rpc.IsSerializable");
                for (TypeMirror extendedInterfaceType : nestedTypeElement.getInterfaces()) {
                    extendedInterfaces.add(extendedInterfaceType.toString());
                }
            
                addLine("public class %s extends %s implements %s {", 
                    nestedTypeElement.getSimpleName(),
                    nestedTypeElement.getSuperclass(),
                    StringUtils.collectionToString(extendedInterfaces, ", ")
                );
                
                increaseIndent();
                
                this.generateBasicClassBody(nestedTypeElement);
                
                decreaseIndent();
                addLine("}");
            }
            
        }
        
        decreaseIndent();
        addLine("}");
    }
    
    private void generateBasicClassBody(TypeElement typeElement) {
        String className = typeElement.getSimpleName().toString();
        
        for (VariableElement field : ElementFilter.fieldsIn(typeElement.getEnclosedElements())) {
            if (field.getModifiers().contains(Modifier.STATIC) || field.getModifiers().contains(Modifier.TRANSIENT)) {
                continue;
            }
            
            addLine("private %s %s;", field.asType(), field.getSimpleName());
            addLine();
        }
        
        // Constructor
        addLine("public " + className + "() { }");
        addLine();

        for (VariableElement field : ElementFilter.fieldsIn(typeElement.getEnclosedElements())) {
            if (field.getModifiers().contains(Modifier.STATIC) || field.getModifiers().contains(Modifier.TRANSIENT)) {
                continue;
            }
            
            String fieldName = field.getSimpleName().toString();
            
            // Setter
            addLine("public void set%s(%s %s) {", StringUtils.initialUpperCase(fieldName), field.asType(), fieldName);
            increaseIndent();
            
            addLine("this.%s = %s;", fieldName, fieldName);
            decreaseIndent();
            addLine("}");
            addLine();
            
            // Getter
            String getMethodName;
            PrimitiveType booleanPrimitiveType = this.types.getPrimitiveType(TypeKind.BOOLEAN);
            TypeMirror booleanObjectType = this.types.boxedClass(booleanPrimitiveType).asType();
            if (this.types.isSameType(booleanPrimitiveType, field.asType()) 
                    || this.types.isSameType(booleanObjectType, field.asType())) {
                getMethodName = "is" + StringUtils.initialUpperCase(fieldName);
            }
            else {
                getMethodName = "get" + StringUtils.initialUpperCase(fieldName);
            }

            addLine("public %s %s() {", field.asType(), getMethodName);
            increaseIndent();
            addLine("return this.%s;", fieldName);
            decreaseIndent();
            addLine("}");
            addLine();
        }

    }
    
}
