/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/logging/JdkLoggerAdapter.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.logging;

import java.util.EnumMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * "Fallback" Log adapter for JDK logging
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class JdkLoggerAdapter implements LogAdapter {
    private static final Map<LogLevel, Level> LEVELS;
    private final Logger logger;
    
    static {
        LEVELS = new EnumMap<LogLevel, Level>(LogLevel.class);
        LEVELS.put(LogLevel.TRACE, Level.FINEST);
        LEVELS.put(LogLevel.DEBUG, Level.FINE);
        LEVELS.put(LogLevel.INFO, Level.INFO);
        LEVELS.put(LogLevel.PROGRESS, Level.INFO);
        LEVELS.put(LogLevel.WARN, Level.WARNING);
        LEVELS.put(LogLevel.ERROR, Level.SEVERE);
        LEVELS.put(LogLevel.FATAL, Level.SEVERE);
    }
    
    JdkLoggerAdapter(Logger logger) {
        this.logger = logger;
    }
    
    public void log(LogMessage msg) {
        log(msg.getLevel(), msg.localizedMessage());
    }
    
    public void log(LogMessage msg, Throwable t) {
        log(msg.getLevel(), msg.localizedMessage(), t);
    }

    public void log(LogLevel level, String msg) {
        this.logger.log(level(level), msg);
    }

    public void log(LogLevel level, String msg, Throwable t) {
        this.logger.log(level(level), msg, t);
    }

    /**
     * Convert the supplied LogLevel into the local equivilent
     * @param level
     * @return
     */
    private static Level level(LogLevel level) {
        Level jdk = LEVELS.get(level);
        return (jdk != null) ? jdk : Level.FINE;
    }   
}
