/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/logging/Log4jAdapter.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.logging;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.EnumMap;
import java.util.Map;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * LogAdapter for Log4j
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class Log4jAdapter implements LogAdapter {
    private static final Map<LogLevel, Level> LEVELS;
    static {
        LEVELS = new EnumMap<LogLevel, Level>(LogLevel.class);
        Level trace = Level.DEBUG;
        try {
            // Older versions of Log4j don't have a TRACE setting
            // Try to find the TRACE static field, if it's not there
            // fall back to debug
            Field f = Level.class.getField("TRACE");
            if (Modifier.isStatic(f.getModifiers()) && Level.class.isAssignableFrom(f.getType())) {
                trace = (Level)f.get(null);
            }
        }
        catch (NoSuchFieldException e) { }
        catch (ClassCastException e) { }
        catch (IllegalAccessException e) { }
        
        LEVELS.put(LogLevel.TRACE, trace);
        LEVELS.put(LogLevel.DEBUG, Level.DEBUG);
        LEVELS.put(LogLevel.INFO, Level.INFO);
        LEVELS.put(LogLevel.PROGRESS, Level.INFO);
        LEVELS.put(LogLevel.WARN, Level.WARN);
        LEVELS.put(LogLevel.ERROR, Level.ERROR);
        LEVELS.put(LogLevel.FATAL, Level.FATAL);
    }
    
    private final Logger logger;
    
    Log4jAdapter(Logger logger) {
        this.logger = logger;
    }
    
    public void log(LogMessage msg) {
        log(msg.getLevel(), msg.localizedMessage());
    }
    
    public void log(LogMessage msg, Throwable t) {
        log(msg.getLevel(), msg.localizedMessage(), t);
    }

    public void log(LogLevel level, String msg) {
        this.logger.log(level(level), msg);
    }

    public void log(LogLevel level, String msg, Throwable t) {
        this.logger.log(level(level), msg, t);
    }
    
    private static Level level(LogLevel level) {
        Level log4jLevel = LEVELS.get(level);
        return (log4jLevel != null) ? log4jLevel : Level.DEBUG;
    }


}
