/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/FileSecurityDescriptor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform;

import java.util.Set;

public interface FileSecurityDescriptor {
    enum AccessControl {
        OWNER_READ,
        OWNER_WRITE,
        OWNER_EXECUTE,
        GROUP_READ,
        GROUP_WRITE,
        GROUP_EXECUTE,
        ALL_READ,
        ALL_WRITE,
        ALL_EXECUTE
    }
    
    /**
     * Returns an object representing the user/group that owns currently owns the file.
     * @return A FileOwner object containing the user and group that own the file
     */
    FileOwner getFileOwner();
    
    /**
     * Grants the specified permissions if they do not already exist.  This call will not remove any existing permissions, 
     * it will only add additional permissions.
     *    
     * @param accessControls The list of permissions to add
     */
    void addAccessControls(AccessControl...accessControls);

    /**
     * Removes the specified permissions if they exist. 
     * 
     * @param accessControls The list of permissions to remove
     */
    void removeAccessControls(AccessControl...accessControls);

    /**
     * Returns the set of access controls that are currently enabled.  This method may result in 
     * native method calls depending on the implementation.
     * 
     * @return The set of enabled access controls.
     */
    Set<AccessControl> getEnabledAccessControls();
}
