/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/sunos/OperatingSystemImpl.java#1 $
 *
 * Copyright (c) 2006 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.sunos;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellRunnable;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.FileSystemMount;
import com.netapp.common.platform.PartitionTable;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.util.FileUtils;

/**
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class OperatingSystemImpl extends com.netapp.common.platform.unix.OperatingSystemImpl {
    private static final NALogger logger = NALogger.getLogger(OperatingSystemImpl.class);
    
    public static final String SOLARIS_PLATFORM_NAME = "sunos";
    
    /**
     * /dev/dsk/c1t1d0s0       /       ufs     rw,intr,largefiles,onerror=panic,suid,dev=1d80008       1233945996
     * pufferfish:/vol/cruise_control_Dnewport_news_R10        /mnt/cruise_control/Dnewport_news_R1.0  nfs     rw,bg,noquota,intr,dev=4f40006  1233946014
     */
    private static final Pattern MNTTAB_PATTERN = 
        Pattern.compile("(?:(\\S+):)?(\\S+)\\s+(\\S+)\\s+(\\S+)\\s+(\\S+)\\s+\\d+.*");
    
    private static final int MNTTAB_SERVER_GROUP = 1;
    private static final int MNTTAB_DEVICE_GROUP = 2;
    private static final int MNTTAB_MOUNT_POINT_GROUP = 3;
    private static final int MNTTAB_TYPE_GROUP = 4;
    private static final int MNTTAB_OPTIONS_GROUP = 5;
    /**
     *
     */
    public OperatingSystemImpl() {
        super();
    }

    @Override
    public String getLibraryEnvironmentName() {    
        return "LD_LIBRARY_PATH";
    }

    @Override
    public DeviceAnalyzer createDeviceAnalyzer() {
        return new SolarisDeviceAnalyzer();
    }

    @Override
    protected ShellRunnable createDFCommand(List<File> fileList) {
        ShellCommand shellCommand = new ShellCommand();
        shellCommand.append("df", "-g");
        for (File file : fileList) {
            shellCommand.append(file.getAbsolutePath());
        }
        return shellCommand;
    }

    @Override
    protected DFResultConstructor createDFResultConstructor() {
        return new com.netapp.common.platform.sunos.DFResultConstructor();
    }

    @Override
    public Collection<FileSystemMount> getMountedFilesystems() {
        List<Matcher> mnttab = null;
        try {
            mnttab = FileUtils.matchLines(MNTTAB_PATTERN, getMountTableFile());
        }
        catch (IOException e) {
            throw new PlatformException(
                logger.debug(MsgKey.PLATFORM_MOUNT_TABLE_READ_FAILED, e.getLocalizedMessage()),
                e);
        }
        
        List<FileSystemMount> results = new ArrayList<FileSystemMount>();
        for (Matcher m : mnttab) { 
            String type = m.group(MNTTAB_TYPE_GROUP);
            if ("autofs".equals(type)) {
                continue;
            }
            String server = m.group(MNTTAB_SERVER_GROUP);
            String device = m.group(MNTTAB_DEVICE_GROUP);
            String options = m.group(MNTTAB_OPTIONS_GROUP);
            String mountPoint = m.group(MNTTAB_MOUNT_POINT_GROUP);
            results.add(new FileSystemMount(server, device, mountPoint, type, options));
        }
        return results;
    }
    
    /**
     * Unit test hook
     */
    protected File getMountTableFile() {
        return new File("/etc/mnttab");
    }
    
    @Override
    public PartitionTable readPartitionTable(File device) {
        try {
            return new SolarisPartitionTable(device);
        }
        catch (IOException e) {
            throw new PlatformException(e);
        }
    }
}
