/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/sunos/SolarisPartitionTable.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */

/*
 * This is a port from the Sun Disklabel code in FreeBSD.
 * The original copyright to the FreeBSD code is reproduced below
 */

/*-
 * Copyright (c) 1992, 1993
 *      The Regents of the University of California.  All rights reserved.
 * Copyright (c) 2004,2005 Joerg Wunsch
 *
 * This software was developed by the Computer Systems Engineering group
 * at Lawrence Berkeley Laboratory under DARPA contract BG 91-66 and
 * contributed to Berkeley.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      @(#)sun_disklabel.h     8.1 (Berkeley) 6/11/93
 *      $NetBSD: disklabel.h,v 1.2 1998/08/22 14:55:28 mrg Exp $
 *
 * $FreeBSD: src/sys/sys/sun_disklabel.h,v 1.11 2005/03/30 09:33:10 joerg Exp $ 
 */

package com.netapp.common.platform.sunos;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.Partition;
import com.netapp.common.platform.PartitionTable;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.util.Utils;

/**
 * This is a port from the Sun Disklabel code in FreeBSD
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class SolarisPartitionTable extends PartitionTable {
    
    private static final NALogger logger = NALogger.getLogger(SolarisPartitionTable.class);
    
    private static final String UTF8 = "UTF-8";

    private static final int SUN_NPART = 8;
    private static final int SUN_SIZE = 512;
    private static final int SUN_VTOC_SANE = 0x600DDEEE;
    private static final int SUN_VOLNAME_LEN = 8;
    private static final int SUN_DKMAGIC = 55998; // This one isn't hex

    private static final int VTOC_UNASSIGNED = 0x00;
    private static final int VTOC_BOOT = 0x01;
    private static final int VTOC_ROOT = 0x02;
    private static final int VTOC_SWAP = 0x03;
    private static final int VTOC_USR = 0x04;
    private static final int VTOC_BACKUP = 0x5;
    private static final int VTOC_STAND = 0x06;
    private static final int VTOC_VAR = 0x07;
    private static final int VTOC_HOME = 0x08;
    private static final int VTOC_ALTSCTR = 0x09;
    private static final int VTOC_CACHE = 0x0a;
    private static final int VTOC_VXVM_PUB = 0x0e;
    private static final int VTOC_VXVM_PRIV = 0x0f;

    private static final String[] VTOC_thisS = {
        "unassigned",
        "boot",
        "root",
        "swap",
        "usr",
        "backup",
        "stand",
        "var",
        "home",
        "altsctr",
        "cache" };

    private static final int VTOC_UNMNT = 0x01;
    private static final int VTOC_RDONLY = 0x10;

    /*
     * This is ported from the Sun Diskthis code in FreeBSD.
     */
    private static final int SL_TEXT = 0x00;
    private static final int SL_TEXT_SIZEOF = 0x080;
    private static final int SL_VTOC_VERS = 0x80;
    private static final int SL_VTOC_VOLNAME = 0x84;
    private static final int SL_VTOC_NPART = 0x8c;
    private static final int SL_VTOC_MAP = 0x8e;
    private static final int SL_VTOC_SANITY = 0xbc;
    private static final int SL_RPM = 0x1a4;
    private static final int SL_PCYLINDERS = 0x1a6;
    private static final int SL_SPARESPECYL = 0x1a8;
    private static final int SL_INTERLEAVE = 0x1ae;
    private static final int SL_NCYLINDERS = 0x1b0;
    private static final int SL_ACYLINDERS = 0x1b2;
    private static final int SL_NTRACKS = 0x1b4;
    private static final int SL_NSECTORS = 0x1b6;
    private static final int SL_PART = 0x1bc;
    private static final int SL_MAGIC = 0x1fc;
    private static final int SL_CKSUM = 0x1fe;

    private static final int SDKP_CYLOFFSET = 0x00;
    private static final int SDKP_NSECTORS = 0x04;
    private static final int SDKP_SIZEOF = 0x08;

    private static final int SVTOC_TAG = 0x00;
    private static final int SVTOC_FLAG = 0x02;
    private static final int SVTOC_SIZEOF = 0x04;

    /*
     * The names on these are from C
     */
    private String text;
    private int rpm;
    private int pcylinders;
    private int sparespercyl;
    private int interleave;
    private int ncylinders;
    private int acylinders;
    private int ntracks;
    private int nsectors;
    private int magic;

    private long vtocSane;
    private long vtocVers;
    private String vtocVolName;


    static class DKPart {
        /** Starting cylinder */
        long cyloffset; 

        /** Number of sectors */
        long nsectors;
    }

    static class VTocInfo {
        /** Partition Tag */
        int tag;

        /** Partition flags */
        int flag;
    }
    
    private VTocInfo[] vtoc;
    private DKPart[] part;

    public SolarisPartitionTable(File device) throws IOException {
        String basePath = device.getAbsolutePath();
        if (basePath.matches("(.*)s[0-9]$")) {
            basePath = basePath.substring(0, basePath.lastIndexOf('s'));
        }
        
        File readableDevice = new File(basePath + "s2");
        RandomAccessFile file = new RandomAccessFile(readableDevice, "r");
        FileChannel channel = file.getChannel();

        byte[] buf = new byte[SUN_SIZE];
        ByteBuffer byteBuffer = ByteBuffer.wrap(buf);
        channel.read(byteBuffer);

        this.text = new String(buf, SL_TEXT, SL_TEXT_SIZEOF, UTF8);

        this.rpm = Utils.unsigned16BE(buf, SL_RPM);
        this.pcylinders = Utils.unsigned16BE(buf, SL_PCYLINDERS);
        this.sparespercyl = Utils.unsigned16BE(buf, SL_SPARESPECYL);
        this.interleave = Utils.unsigned16BE(buf, SL_INTERLEAVE);
        this.ncylinders = Utils.unsigned16BE(buf, SL_NCYLINDERS);
        this.acylinders = Utils.unsigned16BE(buf, SL_ACYLINDERS);
        this.ntracks = Utils.unsigned16BE(buf, SL_NTRACKS);
        this.nsectors = Utils.unsigned16BE(buf, SL_NSECTORS);

        this.part = new DKPart[SUN_NPART];
        for (int i = 0; i < SUN_NPART; i++) {
            this.part[i] = new DKPart();
            this.part[i].cyloffset = Utils.unsigned32BE(buf, SL_PART + (i * SDKP_SIZEOF) + SDKP_CYLOFFSET);
            this.part[i].nsectors = Utils.unsigned32BE(buf, SL_PART + (i * SDKP_SIZEOF) + SDKP_NSECTORS);
        }

        this.magic = Utils.unsigned16BE(buf, SL_MAGIC);

        long vtocsane = Utils.unsigned32BE(buf, SL_VTOC_SANITY);
        int npart = Utils.unsigned16BE(buf, SL_VTOC_NPART);

        if (vtocsane == SUN_VTOC_SANE && npart == SUN_NPART) {
            /*
             * Seems we've got SVR4 compatible VTOC information
             * as well, decode it.
             */
            this.vtocSane = vtocsane;
            this.vtocVers = Utils.unsigned32BE(buf, SL_VTOC_VERS);
            this.vtocVolName = new String(buf, SL_VTOC_VOLNAME, SUN_VOLNAME_LEN, UTF8);

            this.vtoc = new VTocInfo[SUN_NPART];
            for (int i = 0; i < SUN_NPART; i++) {
                this.vtoc[i] = new VTocInfo();
                this.vtoc[i].tag = Utils.unsigned16BE(buf, SL_VTOC_MAP + (i * SVTOC_SIZEOF) + SVTOC_TAG);
                this.vtoc[i].flag = Utils.unsigned16BE(buf, SL_VTOC_MAP + (i * SVTOC_SIZEOF) + SVTOC_FLAG);
            }
        }

        int chksum = 0;
        for (int i = 0; i < SUN_SIZE; i += 2) {
            chksum ^= Utils.unsigned16BE(buf, i);
        }
        if (chksum != 0 || this.magic != SUN_DKMAGIC) {
            if ("x86".equalsIgnoreCase(System.getProperty("os.arch"))) {
                // generating a better error message for the unsupported partition table format                
                throw new PlatformException(logger.error(MsgKey.PLATFORM_UNSUPPORTED_PARTITION_TABLE_FORMAT, device));
            }
            else {
                throw new PlatformException(logger.error(MsgKey.PLATFORM_INVALID_DISK_LABEL_CHECKSUM, device));
            }
        }
        
        /*
         * Now construct our Partition objects from the data structures read from the disk
         */
        long blocksPerCylinder = this.ntracks * this.nsectors;
        for (int i = 0; i < this.part.length; i++) {
            File devicePartition = new File(String.format("%ss%d", basePath, i));
            long start = this.part[i].cyloffset * blocksPerCylinder * SUN_SIZE;
            long size = this.part[i].nsectors * SUN_SIZE;
            
            this.addPartition(new Partition(devicePartition, start, size));
        }
        
        ByteArrayOutputStream stringOutputStream = new ByteArrayOutputStream();
        this.print(new PrintStream(stringOutputStream));
        
        logger.debug(MsgKey.PLATFORM_PARTITION_TABLE, device, stringOutputStream);
    }

    public void print(PrintStream out) {
        out.println(this.text);
        out.printf("Physical Cylinders: %d + %d (reserved cylinders)\n", 
                this.ncylinders, 
                this.acylinders);

        out.println("ncyl " + this.ncylinders);
        out.println("acyl " + this.acylinders);
        out.println("ntrk " + this.ntracks);
        out.println("nsec " + this.nsectors);

        // Blocks per cylinder
        long bpc = this.ntracks * this.nsectors;
        for (int i = 0; i < this.part.length; i++) {
            long size = (this.part[i].nsectors * SUN_SIZE) / (1024 * 1024);
            long cyls = this.part[i].nsectors / bpc;
            String tag = "";
            String flag = "";

            if (this.vtoc != null) {
                if (VTOC_thisS.length > this.vtoc[i].tag) {
                    tag = VTOC_thisS[this.vtoc[i].tag];
                }
                else {
                    tag = "unkwn";
                }

                flag = ((this.vtoc[i].flag & VTOC_RDONLY) != 0) ? "r" : "w";
                flag += ((this.vtoc[i].flag & VTOC_UNMNT) != 0) ? "u" : "m";
            }

            /* Go the other way to find byte offset for SMO
             * Multiple cyloffset * bpc * SUN_SIZE
             */
            long end = (cyls > 0) ? this.part[i].cyloffset + cyls - 1 : 0;

            out.printf("%3d %8s %2s %5d  - %5d %5dMB %10d\n", 
                    i, 
                    tag,
                    flag,
                    this.part[i].cyloffset,
                    end,
                    size,
                    this.part[i].nsectors);
        }
    }

    /**
     * @see com.netapp.common.platform.PartitionTable#findPartition(int)
     */
    @Override
    public Partition findPartition(int partitionNumber) {
        // Solaris partition numbers start at 0
        return this.getPartitions().get(partitionNumber);
    }
}
