/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/unix/DFResultConstructor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.unix;

import java.io.File;
import java.util.List;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;

public abstract class DFResultConstructor {
    
    private static final NALogger logger = NALogger.getLogger(DFResultConstructor.class);
    private int linesPerFile;
    private int filesStartIndex;

    public DFResultConstructor(int filesStartIndex, int linesPerFile) {
        this.filesStartIndex = filesStartIndex;
        this.linesPerFile = linesPerFile;
    }
    
    protected abstract DFResult doConstruct(List<File> fileList, ShellResult shellResult);
    
    final DFResult construct(List<File> fileList, ShellResult shellResult) {
        if (!shellResult.getStdErr().isEmpty()) {
            throw new PlatformException(logger.error(MsgKey.PLATFORM_EXECUTION_ERROR_EXECUTING_D_F_COMMAND, 
                shellResult.getStdErr()));
        }
        
        if (shellResult.isNonZeroExitCode()) {
            throw new PlatformException(logger.error(MsgKey.PLATFORM_EXECUTION_ERROR_EXECUTING_D_F_COMMAND, 
                shellResult.getAllOut()));
        }
        
        if (shellResult.getStdOut().size() - this.filesStartIndex < fileList.size() * this.linesPerFile) {
            throw new PlatformException(logger.error(MsgKey.PLATFORM_EXECUTION_MISSING_DF_RECORDS,
                fileList.size(), (shellResult.getStdOut().size() - this.filesStartIndex) / this.linesPerFile));
        }
        
        return this.doConstruct(fileList, shellResult);
    }
    
    
}
