/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/spring/SingletonScope.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.spring;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;

/**
 * A singleton scope implementation.
 * 
 * Spring already provides its own built-in singleton scope, so this
 * is only really useful for aliasing another scope keyword to a singleton scope.
 * 
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class SingletonScope implements Scope {

    private Map<String, Object> singletonMap;
    
    public SingletonScope() {
        this.singletonMap = new HashMap<String, Object>();
    }
    
    public synchronized Object get(String name, ObjectFactory objectFactory) {
        Object object = this.singletonMap.get(name);
        
        if (object == null) {
            object = objectFactory.getObject();
            this.singletonMap.put(name, object);
        }
        
        return object;
    }

    public synchronized Object remove(String name) {
        return this.singletonMap.remove(name);
    }

    public String getConversationId() {
        return null;
    }

    public void registerDestructionCallback(String name, Runnable callback) {
        // Unsupported
    }

}
